/*
 Analog VU meter / Clock
 
 Instructables Link: http://www.instructables.com/id/Analog-VU-meter-and-Clock-Arduino-Powered/ 
 
 Credits:
  bildr - DS1307 RTC code http://bildr.org/2011/03/ds1307-arduino/
 _____________________
 Wiring goes as follows:

 - USB red wire (5v) to VCC
 - USB black wire to GND

 - 3.5mm Audio left channel to 10K resistor to Analog 1
 - 3.5mm Audio right channel to 10K resistor to Analog 2
 - 3.5mm Audio ground to GND

 - Potentiometer to Analog 0 (follow arduino tutorial: http://arduino.cc/en/uploads/Tutorial/graph-circuit3.png )

 - Buttons - Left/Down to Digital 2  (follow arduino tutorial: http://arduino.cc/en/uploads/Tutorial/button.png )
           - Right/Up to Digital 3

 - DS1307 RTC  - SDA to Analog 4
               - SCL to Analog 5

 - Left Analog Meter to Digital 5 (PWM)
 - Right Analog Meter to Digital 6 (PWM)
_______________________

 Note:
  Because all meters are slightly different, the PWM values 
  need to be adjusted based on your design. Feel free to use 
  my code, but you will have to change some values because 
  it is set to work specifically with my meters.
 
 */

#include "Wire.h"
#define DS1307_ADDRESS 0x68
byte zero = 0x00; //workaround for issue #527

int downButton = 2;    // down push button to adjust time (pin 2)
int upButton = 3;      // up push button to adjust time (pin 3)

int hours = 0;         // clock variables
int minutes = 0;
int seconds = 0;
int lastMinute = 0;
long clockWait = 0;    
int clockMode = 0;

int leftMeter = 5;       // left meter (hours) is attached to pin 5 (PWM)
int rightMeter = 6;     // right meter (min) is attached to pin 6 (PWM)
int leftChannel = 0;     // left channel of audio (analog 1)
int rightChannel = 0;    // right channel of audio (analog 2)
int sensitivity = 0;     // 10k pot sensitivity control (analog 0)
int rightLevel = 0;      // position of the right meter
int leftLevel = 0;       // position of the left meter 
// rtc - SDA to A4 - SCL to A5
 
void setup()  { 
  Wire.begin();
  // Serial.begin(9600); Un-comment to view time in serial monitor
  
  pinMode(leftMeter, OUTPUT);      // Initialize Outputs 
  pinMode(rightMeter, OUTPUT);
  
  digitalWrite(leftMeter, HIGH);   // Test meters on startup
  digitalWrite(rightMeter, HIGH);
  digitalWrite(13, HIGH);
  delay (1000);
  digitalWrite(leftMeter, LOW);
  digitalWrite(rightMeter, LOW);
  digitalWrite(13, LOW);
  delay(1000);
 } 


void loop()  { 
  
  sensitivity = (analogRead(A0) / 3 + 1); // potentiometer sensitivity control
    if (clockMode == 0){
     analogWrite(leftMeter, leftLevel); // adjust left meter level
     analogWrite(rightMeter, rightLevel); // adjust right meter level
    }
     
     leftChannel = analogRead(A1); // read left channel of audio 
     rightChannel = analogRead(A2); // read right channel of audio

   
  // Below is the code to set meter levels based on sound levels: 
  // If you change the values, remember to make sure that the Left and Right channels are matching.
  // leftLevel and rightLevel can be set from 0 to 255 (PWM)
  // (clock code starts at line 204)
  
    if (leftChannel >= sensitivity && leftLevel < 20){   
   leftLevel = 20; 
      }
      
       if (rightChannel >= sensitivity && rightLevel < 20){
     rightLevel = 20;
        }
    
    if (leftChannel >= sensitivity + 5 && leftLevel < 41){   
   leftLevel = 41;
      }
      
       if (rightChannel >= sensitivity + 5 && rightLevel < 41){
     rightLevel = 41;
        }
        
         if (leftChannel >= sensitivity + 10 && leftLevel < 62){   
   leftLevel = 62;
      }
      
       if (rightChannel >= sensitivity + 10 && rightLevel < 62){
     rightLevel = 62;
        }
      
    if (leftChannel >= sensitivity + 15 && leftLevel < 83){   
   leftLevel = 83;
      }
      
      if (rightChannel >= sensitivity + 15 && rightLevel < 83){
     rightLevel = 83;
        }
        
         if (leftChannel >= sensitivity + 20 && leftLevel < 104){   
   leftLevel = 104;
      }
      
       if (rightChannel >= sensitivity + 20 && rightLevel < 104){
     rightLevel = 104;
        }
        
    if (leftChannel >= sensitivity + 25 && leftLevel < 125){
   leftLevel = 125;
      }
      
        if (rightChannel >= sensitivity + 25 && rightLevel < 125){
     rightLevel = 125;
      }
    
     if (leftChannel >= sensitivity + 30 && leftLevel < 146){   
   leftLevel = 146;
      }
      
       if (rightChannel >= sensitivity + 30 && rightLevel < 146){
     rightLevel = 146;
        }
    
    if (leftChannel >= sensitivity + 35 && leftLevel < 167){
   leftLevel = 167;
       } 
       
        if (rightChannel >= sensitivity + 35  && rightLevel < 167){
      rightLevel = 167;
       }
       
    if (leftChannel >= sensitivity + 37 && leftLevel < 188){
   leftLevel = 188;
       } 
       
       if (rightChannel >= sensitivity + 37  && rightLevel < 188){
      rightLevel = 188;
       }
     
     if (leftChannel >= sensitivity + 40 && leftLevel < 209){
    leftLevel = 209;
       }  
         
         if (rightChannel >= sensitivity + 40 && rightLevel < 209){
       rightLevel = 209;
       }
       
         if (leftChannel >= sensitivity + 42 && leftLevel < 230){
    leftLevel = 230;
       }  
         
         if (rightChannel >= sensitivity + 42 && rightLevel < 230){
       rightLevel = 230;
       }
 
      if (leftChannel >= sensitivity + 45 && leftLevel < 255){
    leftLevel = 255;
       }
  
         if (rightChannel >= sensitivity + 45 && rightLevel < 255){
       rightLevel = 255;
       }

 // Fade the meters out to 0
   if (leftChannel < sensitivity && leftLevel > 0){   
     leftLevel = --leftLevel;  // left meter                       
     delayMicroseconds(1000); // can be increased and decreased to adjust smoothness 
   }                          // (how long it takes to get back to 0)
    
     if (rightChannel < sensitivity && rightLevel > 0){    
       rightLevel = --rightLevel;  // right meter
      delayMicroseconds(1000);    // can be increased and decreased to adjust smoothness 
    }                             // (how long it takes to get back to 0)
  
 
// Clock code below -- Credit to bildr.org for most of the code

 if (leftLevel == 0 && rightLevel == 0 || leftLevel == 255 && rightLevel == 255){
    if (millis() - clockWait >= 6000){  // wait 6 seconds to make sure no sound is detected
clockMode = 1; // change to clock mode
     }
    }else{
  clockWait = millis(); // if sound is detected turn off clock mode
  clockMode = 0;
     }
     
if (clockMode == 1){
       printDate(); 
     }
   
 // Changing the time with buttons  
  if (digitalRead(downButton) == HIGH && clockMode == 1){  // decrease time
 byte second = 0;
 byte minute = minutes; // Don't get confused: minute is not the same as minutes
 byte hour = hours;     // Don't get confused: hour is not the same as hours
 delay(75);
if (minutes > 0){ 
 --minute; 
}else{
  minute = 59;
 if (hours > 0){ 
  -- hour;
 }else{
   hour = 23;
}}
 Wire.beginTransmission(DS1307_ADDRESS);
 Wire.write(zero); //stop Oscillator
  
  Wire.write(decToBcd(second));
  Wire.write(decToBcd(minute));
  Wire.write(decToBcd(hour));
 
  Wire.write(zero); //start 
  
  Wire.endTransmission();
}
 
 if (digitalRead(upButton) == HIGH && clockMode == 1){  // increase time
 byte second = 0;
 byte minute = minutes; // Don't get confused: minute is not the same as minutes
 byte hour = hours;     // Don't get confused: hour is not the same as hours
 delay(75);
if (minutes < 59){ 
 ++minute; 
}else{
  minute = 0;
 if (hours < 23){ 
  ++ hour;
 }else{
   hour = 0;
}}
 Wire.beginTransmission(DS1307_ADDRESS);
 Wire.write(zero); //stop Oscillator
  
  Wire.write(decToBcd(second));
  Wire.write(decToBcd(minute));
  Wire.write(decToBcd(hour));
 
  Wire.write(zero); //start 
  
  Wire.endTransmission();
}
  }

byte decToBcd(byte val){
// Convert normal decimal numbers to binary coded decimal
  return ( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val)  {
// Convert binary coded decimal to normal decimal numbers
  return ( (val/16*10) + (val%16) );
}

void printDate(){
  // Reset the register pointer
  Wire.beginTransmission(DS1307_ADDRESS);
  Wire.write(zero);
  Wire.endTransmission();

  Wire.requestFrom(DS1307_ADDRESS, 7);
  
  seconds = bcdToDec(Wire.read());
  minutes = bcdToDec(Wire.read());
  hours = bcdToDec(Wire.read() & 0b111111); //24 hour time
  
  // Serial.print(hours);     // Un-comment to view time in serial monitor  
  // Serial.print(":");       //
  // Serial.println(minutes); // Don't forget to un-comment 'Serial.begin' in 'void setup' above

// Specific meter levels for each hour 
// You will likly have to adjust these values.
// Play around with the values until you get what you want.
   if (hours == 1 || hours == 13){
     analogWrite(leftMeter, 0); // adjust left (hour) meter level
      }
    if (hours == 2 || hours == 14){
     analogWrite(leftMeter, 29); // adjust left (hour) meter level
      }
      if (hours == 3 || hours == 15){
     analogWrite(leftMeter, 52); // adjust left (hour) meter level
      }
       if (hours == 4 || hours == 16){
     analogWrite(leftMeter, 78); // adjust left (hour) meter level
      }
      if (hours == 5 || hours == 17){
     analogWrite(leftMeter, 104); // adjust left (hour) meter level
      }
       if (hours == 6 || hours == 18){
     analogWrite(leftMeter, 130); // adjust left (hour) meter level
      }
      if (hours == 7 || hours == 19){
     analogWrite(leftMeter, 156); // adjust left (hour) meter level
      }
      if (hours == 8 || hours == 20){
     analogWrite(leftMeter, 175); // adjust left (hour) meter level
      }
      if (hours == 9 || hours == 21){
     analogWrite(leftMeter, 193); // adjust left (hour) meter level
      }
       if (hours == 10 || hours == 22){
     analogWrite(leftMeter, 213); // adjust left (hour) meter level
      }
      if (hours == 11 || hours == 23){
     analogWrite(leftMeter, 234); // adjust left (hour) meter level
      }
       if (hours == 0 || hours == 12){
     analogWrite(leftMeter, 255); // adjust left (hour) meter level
      }
 
 // adjust minute levels (right meter)
  if (minutes * 4.75 < 255){                 // increase PWM value by 4.75 each minute
  analogWrite(rightMeter, minutes * 4.75);   // I had to use 4.75 on my meter to get 30 minutes to point exactly straight up
 }else{                                       // perfect would be 4.25 because 4.25 * 60 = 255
   analogWrite(rightMeter, 255);
 }
    

}
   
 
  
 

